package F5::Policy::Convert::BotProfile;

use strict;
use warnings;

use F5::ASMConfig::GuaranteedClient;
use F5::Utils::File;
use F5::Logger;
use F5::DbUtils;

# Most cBIP Bot Profile actions are not yet available in BD-handled Bot Defense.
# In the interim, unsupported actions will be mapped to Block.
our $BOT_ACTION_MAP = {
               'none' => F5::ASMConfig::BotAction::BOT_ACTION_IGNORE(),
               'alarm' => F5::ASMConfig::BotAction::BOT_ACTION_ALARM(),
               'block' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
               'captcha' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
               'honeypot-page' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
               'redirect-to-pool' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
               'tcp-reset' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
               'rate-limit' => F5::ASMConfig::BotAction::BOT_ACTION_BLOCK(),
};

our $BOT_CLASS_MAP = {
    'Trusted Bot'=> F5::ASMConfig::BotClass::BOT_CLASS_TRUSTED_BOT,
    'Untrusted Bot'=> F5::ASMConfig::BotClass::BOT_CLASS_UNTRUSTED_BOT,
    'Malicious Bot'=> F5::ASMConfig::BotClass::BOT_CLASS_MALICIOUS_BOT,
    'Suspicious Browser'=> F5::ASMConfig::BotClass::BOT_CLASS_SUSPICIOUS_BROWSER,
    'Unknown'=> F5::ASMConfig::BotClass::BOT_CLASS_UNKNOWN,
};

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;

    my $self = {};
    bless $self, $class;
    $self->init(%params);

    return $self;
}

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;

    $self->{logger} = $params{logger} || F5::Logger->new('/ts/log/policy_convert.log');
    $self->{client} = $params{client};
    $self->{dbh} =  $params{dbh} || F5::DbUtils::get_dbh();
    if (! $self->{client}) {
        $self->{client} = F5::ASMConfig::GuaranteedClient::get_client((username => 'import_export_policy'));
    } 
}

#----------------------------------------------------------------------

sub import_bot_profile {
    my($self, %params) = @_;

    my $policy = $params{policy};
    my $bot_profile_json = $params{bot_profile};
    my $call_flags = $params{call_flags};

    $self->{client}->update_bot_defense($self->{client}->{session},
                                     $call_flags,
                                     $policy,
                                     F5::ASMConfig::BotDefense->new({flg_enabled => 1}));

    for my $bot_profile (values %$bot_profile_json) {
        my $flg_is_transparent = ($bot_profile->{'enforcement-mode'} && $bot_profile->{'enforcement-mode'} eq 'transparent') || 0;

        for my $class_override (keys %{$bot_profile->{'class-overrides'}}) {
            my $bot_class = $BOT_CLASS_MAP->{$class_override};
            next unless $bot_class;
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'class-overrides'}->{$class_override}->{mitigation}->{action},
                                            );
            next unless $action;
            my $policy_bot_class = F5::ASMConfig::PolicyBotClass->new({bot_class => $bot_class,
                                                                           action => $action,
                                                                          });
            $self->{client}->update_policy_bot_class($self->{client}->{session},
                                                      $call_flags,
                                                      $policy,
                                                      $policy_bot_class,
                                                     );
        }

        if (defined $bot_profile->{'allow-browser-access'} && $bot_profile->{'allow-browser-access'} eq 'disabled' && $bot_profile->{'browser-mitigation-action'}) {
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'browser-mitigation-action'},
                                            );
            next unless $action;
            my $policy_bot_class = F5::ASMConfig::PolicyBotClass->new({bot_class => F5::ASMConfig::BotClass::BOT_CLASS_BROWSER,
                                                                       action => $action,
                                                                      });
            $self->{client}->update_policy_bot_class($self->{client}->{session},
                                                      $call_flags,
                                                      $policy,
                                                      $policy_bot_class,
                                                     );
        }

        # bigip-next/NAP does not support categories overrides, the override will be to each related signature separately
        for my $sig_category_override (keys %{$bot_profile->{'signature-category-overrides'}}) {
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'signature-category-overrides'}->{$sig_category_override}->{'action'},
                                            );
            next unless $action;
            my @bot_sigs = F5::ASMConfig::Entity::BotSignature->retrieve_all(dbh => $self->{dbh},
                                                                             get_criteria => {category => $sig_category_override},
                                                                            );
            for my $bot_sig (@bot_sigs) {
                my $policy_bot_sig = F5::ASMConfig::PolicyBotSignature->new({bot_signature => $bot_sig,
                                                                             action => $action,
                                                                            });
                $self->{client}->add_policy_bot_signature($self->{client}->{session},
                                                          $call_flags,
                                                          $policy,
                                                          $policy_bot_sig,
                                                         );
            }
        } 

        for my $anomaly_category_override (keys %{$bot_profile->{'anomaly-category-overrides'}}) {
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'anomaly-category-overrides'}->{$anomaly_category_override}->{'action'},
                                            );
            next unless $action;
            my @bot_anomalies = F5::ASMConfig::Entity::BotAnomaly->retrieve_all(dbh => $self->{dbh},
                                                                                get_criteria => {category => $anomaly_category_override},
                                                                               );
            for my $bot_anomaly (@bot_anomalies) {
                my $policy_bot_anomaly = F5::ASMConfig::PolicyBotAnomaly->new({bot_anomaly => $bot_anomaly,
                                                                               action => $action,
                                                                              });
                $self->{client}->add_policy_bot_anomaly($self->{client}->{session},
                                                          $call_flags,
                                                          $policy,
                                                          $policy_bot_anomaly,
                                                         );
            }
        } 

        for my $sig_override (keys %{$bot_profile->{'signature-overrides'}}) {
            my $bot_sig = F5::ASMConfig::Entity::BotSignature->new(dbh => $self->{dbh},
                                                                   get_criteria => {bot_name => $sig_override},
                                                                  );
            next unless $bot_sig;
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'signature-overrides'}->{$sig_override}->{'action'},
                                            );
            next unless $action;
            my $policy_bot_sig = F5::ASMConfig::PolicyBotSignature->new({bot_signature => $bot_sig,
                                                                         action => $action,
                                                                        });
            $self->{client}->add_policy_bot_signature($self->{client}->{session},
                                                      $call_flags,
                                                      $policy,
                                                      $policy_bot_sig,
                                                     );
        }

        for my $anomaly_override (keys %{$bot_profile->{'anomaly-overrides'}}) {
            my $bot_anomaly = F5::ASMConfig::Entity::BotAnomaly->new(dbh => $self->{dbh},
                                                                     get_criteria => {bot_anomaly_name => $anomaly_override},
                                                                    );
            next unless $bot_anomaly;
            my $action = get_bot_action(flg_is_transparent => $flg_is_transparent,
                                             action => $bot_profile->{'anomaly-overrides'}->{$anomaly_override}->{'action'},
                                            );
            next unless $action;
            my $policy_bot_anomaly = F5::ASMConfig::PolicyBotAnomaly->new({bot_anomaly => $bot_anomaly,
                                                                           action => $action,
                                                                          });
            $self->{client}->add_policy_bot_anomaly($self->{client}->{session},
                                                      $call_flags,
                                                      $policy,
                                                      $policy_bot_anomaly,
                                                     );
        }
    }
    return;
}

#----------------------------------------------------------------------

sub get_bot_action {
    my(%params) = @_;

    my $action = $params{action};
    my $flg_is_transparent = $params{flg_is_transparent};

    $action = $BOT_ACTION_MAP->{$action};
    if ($flg_is_transparent && $action && $action == F5::ASMConfig::BotAction::BOT_ACTION_BLOCK()) {
        return F5::ASMConfig::BotAction::BOT_ACTION_ALARM();
    }
    return $action;
}

#----------------------------------------------------------------------

return 1;
