package F5::PrepareConf::Transform;

use strict;
use warnings;


use File::Slurp;
use YAML::XS;
use F5::ASMConfig::Handler::Rest;
use F5::BdCfgTables;
use F5::Deploy;
use F5::DbUtils;

our $GEN_PERL_PREFIX = '__perl_gen__';
our $PROCESSING_DONE_MARKER = '__processing_done';
our $TRANSFORMS_CONFIG_DIR = F5::Deploy::map_path('/usr/share/ts/share/compiler');
our $TRANSFORMS_CONFIG_FILE = "$TRANSFORMS_CONFIG_DIR/transforms.yaml";

my %dcc_table_config_by_name;

#---------------------------------------------------

#For build time
sub process_transforms {
    my (%params) = @_;
    my $dbh = $params{dbh} || F5::DbUtils::get_dbh();
    my $transform_config = YAML::XS::LoadFile($TRANSFORMS_CONFIG_FILE);
    my $transforms_dir = $TRANSFORMS_CONFIG_DIR . '/transforms';
    local $YAML::XS::Boolean = "JSON::PP";

    for my $transform_name (@{$transform_config->{transforms}}) {
        my $file_name = $transforms_dir . "/${transform_name}.yaml";
        my $schema;
        eval {
            $schema = YAML::XS::LoadFile($file_name);
        };
        #For debugging failures
        if (my $err = $@) {
           if ($err =~ /line: ([0-9]+)/) {
               my $line_number = $1;
               my $yaml = read_file($file_name);
               my @lines = (split /\n/, $yaml);
               my $l = "$lines[$line_number - 2]\n$lines[$line_number - 1]\n$lines[$line_number]\n";
               die "$err Line: $l";
           }
           die $err;
        }

        if (!$schema->{$transform_name}->{entities}) {
            #Transform is still in "perl" format
            next;
        }

        my $processed_schema = process_transform(transform => $schema->{$transform_name},
                                                 gen_prefix => $GEN_PERL_PREFIX,
                                                 dbh => $dbh,);

        my $yaml = YAML::XS::Dump({ $transform_name => $processed_schema });
        $yaml =~ s/---\n//; #Remove header
        open (my $fh, '>', $file_name);
        print $fh $yaml;
        close($fh)
    }
}

#---------------------------------------------------

#To process a single transform at build time or dynamically at run_time
sub process_transform {
    my (%params) = @_;
    my $transform = $params{transform};
    my $dbh = $params{dbh};
    my $gen_prefix = $params{gen_prefix} || ''; 
    local $YAML::XS::Boolean = "JSON::PP";

    if (!%dcc_table_config_by_name) {
        %dcc_table_config_by_name = %{F5::BdCfgTables::get_dcc_table_config(as_href_by_key => 'name', dbh => $dbh)};
    }
    my %fields;
    my $entity_type;
    # Add hints for source table
    for my $entity (@{$transform->{entities}}) {
        my ($root_path, @path_parts) = split(/\./, $entity->{policy_path}); # ex. policy.filetypes
        my $path_pointer = $F5::ASMConfig::Handler::Rest::path_mapper->{references}->{policies};
        while (my ($collection_name) = (shift @path_parts)) {
            last unless $collection_name;
            if ($path_pointer->{references} && $path_pointer->{references}->{$collection_name}) {
                $path_pointer = $path_pointer->{references}->{$collection_name};
            } elsif (!$path_pointer->{references}) {    # For Hierarchical paths. Eg: policy.general.allowedResponseCodes
                my $rev_rest_field_mapping = $path_pointer->{entity_type}->reverse_rest_field_map();
                my $collection_ref = $rev_rest_field_mapping->{$collection_name};
                $path_pointer = $path_pointer->{entity_type}->diff_structs()->{$collection_ref};
            }
        }

        $entity_type = $path_pointer->{entity_type};
        $entity->{$gen_prefix . 'table_name'} = $entity_type->db_table();
        $entity->{$gen_prefix . 'db_fields'} = $entity_type->get_db_fields(get_as_transform_field_names => 1);
        $entity->{$gen_prefix . 'entity_type'} = $entity_type;

        if ($entity->{order_by}) {
            my @rest_sort_orders;
            for my $order_by (@{$entity->{order_by}}) {
                my $full_field = $order_by->{source_field};
                my ($field, $nested_fields) = split(/\./, $full_field, 2);
                push (@rest_sort_orders, { full_field => $full_field,
                                            direction => F5::ASMConfig::SortDirection::SORT_DIRECTION_ASC(),
                                            rest_field => $field,
                                            nested_fields => $nested_fields });
            }
            my $sort_orders = $entity_type->convert_rest_orders(dbh => $dbh,
                                                                rest_orders => \@rest_sort_orders);
            my $sort_orders_rv = $entity_type->sort_orders_to_db_clauses(sort_orders => $sort_orders, dbh => $dbh);
            $entity->{$gen_prefix . 'orderby_clause'} = $sort_orders_rv->{orderby_clause};
        }
        process_transform_condition(entity_type => $entity_type,
                                    condition => $entity->{condition},
                                    gen_prefix => $gen_prefix,
                                    dbh       => $dbh,);
    }
    # Add hints for dest table
    for my $bd_cfg_table (@{$transform->{bd_cfg_tables}}) {
        $bd_cfg_table->{$gen_prefix . 'table_name'} = $dcc_table_config_by_name{$bd_cfg_table->{name}}->{table};
        process_transform_condition(entity_type => $entity_type,
                                    condition => $bd_cfg_table->{condition},
                                    gen_prefix => $gen_prefix,
                                    dbh       => $dbh,);
    }
    # Add hints for fields
    # (If we double alias the db_fields, we can list which fields need extra handling for less work)
    for my $field_def (@{$transform->{fields}}) {
        process_transform_condition(entity_type => $entity_type,
                                    condition => $field_def->{condition},
                                    gen_prefix => $gen_prefix,
                                    dbh       => $dbh,);
    }
    $transform->{$PROCESSING_DONE_MARKER} = JSON::true;
    return $transform;
}

#---------------------------------------------------

sub process_transform_condition {
    my (%params) = @_;
    my $entity_type = $params{entity_type};
    my $condition = $params{condition};
    my $gen_prefix = $params{gen_prefix} || ''; 
    my $dbh = $params{dbh};

    return unless $condition && $entity_type;
    for my $comparison (@$condition) {
        next if ($comparison->{source_map}); # Source maps should expect the value to already be in JSON/REST form for comparison

        my $source_field = $comparison->{source_field};
        ## TODO: handle value_lists etc.
        my $field_name = $entity_type->reverse_rest_field_map->{$source_field};
        if ($field_name) {
            $comparison->{$gen_prefix . 'db_value'} = $entity_type->map_rest_field_value( field_value => $comparison->{value},
                                                                                        field_name => $field_name,
                                                                                        rest_field_name => $source_field,
                                                                                        field_def => $entity_type->all_fields->{$field_name},
                                                                                        direction => 'from_rest',
                                                                                        dbh => $dbh,);
        }
    }
}

#---------------------------------------------------

1;
